"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const ow_1 = __importDefault(require("ow"));
const Account_1 = require("../Account");
const AuthAdapter_1 = require("../adapter/AuthAdapter");
const RolesAdapter_1 = require("../adapter/RolesAdapter");
const Mutations_1 = require("./Mutations");
const PrivateActions_1 = require("./PrivateActions");
const RolesAuthModule_1 = require("./RolesAuthModule");
/**
 *
 * Initialize
 */
class Initialize {
    constructor(config, authAdapter) {
        this.config = config;
        this.authAdapter = authAdapter;
    }
    getAction() {
        return ({ commit, dispatch }) => {
            this.authAdapter.initialize({
                onAuthenticated: (user) => this.onAuthenticated(commit, dispatch, user),
                onNotAuthenticated: () => this.onNotAuthenticated(commit),
                onError: (msg) => this.onError(commit, msg),
            });
        };
    }
    onAuthenticated(commit, dispatch, userInfo) {
        const account = Account_1.Account.fromFirebaseUserInfo(userInfo);
        Mutations_1.Mutations.SetAccount.commit(commit, account);
        Mutations_1.Mutations.SetState.commit(commit, { state: RolesAuthModule_1.RolesAuthModule.AuthState.AUTHENTICATED });
        PrivateActions_1.PrivateActions.EnsureAccountRegistered.dispatch(dispatch, account);
        this.config.callbacks.onAuthenticated(account);
    }
    onNotAuthenticated(commit) {
        Mutations_1.Mutations.SetState.commit(commit, { state: RolesAuthModule_1.RolesAuthModule.AuthState.NOTAUTHENTICATED });
        this.config.callbacks.onNotAuthenticated();
    }
    onError(commit, errorMsg) {
        Mutations_1.Mutations.SetState.commit(commit, { state: RolesAuthModule_1.RolesAuthModule.AuthState.NOTAUTHENTICATED });
        this.config.callbacks.onError(errorMsg);
    }
}
/**
 *
 * Logout
 */
class Logout {
    constructor(authAdapter) {
        this.authAdapter = authAdapter;
    }
    getAction() {
        return ({ commit }) => {
            Mutations_1.Mutations.SetState.commit(commit, { state: RolesAuthModule_1.RolesAuthModule.AuthState.LOADING });
            (() => __awaiter(this, void 0, void 0, function* () {
                yield this.authAdapter.signOut();
                Mutations_1.Mutations.SetState.commit(commit, {
                    state: RolesAuthModule_1.RolesAuthModule.AuthState.NOTAUTHENTICATED,
                });
                Mutations_1.Mutations.ResetUser.commit(commit);
            }))();
        };
    }
}
/**
 *
 * CheckRoles
 */
class CheckRole {
    constructor(config, rolesAdapter) {
        this.config = config;
        this.rolesAdapter = rolesAdapter;
    }
    getAction() {
        return ({ commit, state }, role) => {
            (() => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.validateInput(role, state.account);
                    const hasRole = yield this.doCheckRole(state.account.uid, role);
                    Mutations_1.Mutations.SetRole.commit(commit, { role, hasRole });
                    if (!hasRole) {
                        const isRequestingRole = yield this.doCheckRoleRequest(state.account.uid, role);
                        Mutations_1.Mutations.SetRoleRequest.commit(commit, { role, isRequestingRole });
                    }
                }
                catch (error) {
                    this.config.callbacks.onError(`Could not ensure user is registered: ${error.message}`);
                }
            }))();
        };
    }
    doCheckRole(uid, role) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.rolesAdapter.hasRole(uid, role);
        });
    }
    doCheckRoleRequest(uid, role) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.rolesAdapter.isRoleRequestedByUser(uid, role);
        });
    }
    validateInput(role, account) {
        ow_1.default(role, "role", ow_1.default.string.oneOf(_.keys(this.config.roles.roles)));
        if (!account)
            throw new Error("Cannot get role before authentication");
    }
}
/**
 *
 * EnsureAccountRegistered
 */
class EnsureAccountRegistered {
    constructor(config, rolesAdapter) {
        this.config = config;
        this.rolesAdapter = rolesAdapter;
    }
    getAction() {
        return ({}, user) => {
            (() => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.doEnsureRegistered(user);
                }
                catch (error) {
                    this.config.callbacks.onError(`Could not ensure user is registered: ${error.message}`);
                }
            }))();
        };
    }
    doEnsureRegistered(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const userExists = yield this.rolesAdapter.userExists(user.uid);
            if (!userExists) {
                yield this.rolesAdapter.registerUser(user);
            }
        });
    }
}
/**
 *
 * Export of actions
 */
function constructActions(config, auth, firestore) {
    const rolesAdapter = new RolesAdapter_1.RolesAdapter(config, firestore);
    const authAdapter = new AuthAdapter_1.AuthAdapter(auth);
    return {
        [RolesAuthModule_1.RolesAuthModule.Actions.Initialize.name]: new Initialize(config, authAdapter).getAction(),
        [RolesAuthModule_1.RolesAuthModule.Actions.Logout.name]: new Logout(authAdapter).getAction(),
        [RolesAuthModule_1.RolesAuthModule.Actions.CheckRole.name]: new CheckRole(config, rolesAdapter).getAction(),
        [PrivateActions_1.PrivateActions.EnsureAccountRegistered.name]: new EnsureAccountRegistered(config, rolesAdapter).getAction(),
    };
}
exports.constructActions = constructActions;
//# sourceMappingURL=impl_actions.js.map